
--[[ string constants ]]--
--[[ put all the strings that might require translation in ]]--
--[[ one location for easier access ]]--
helpString = "Travel\n\ntrav show: Display the Travel Window\n" ..
            "trav hide: Hides the Travel Window\n" ..
            "trav toggle: Toggle the Travel Window\n";

-- window title strings
mainTitleString = "Travel";
optionsTitleString = "Travel Options";

-- strings for maps
mapHomeString = "Map Home";
glanMapString = "Glân Vraig Map";
glanMapItemString = "Map to Glân Vraig";

-- travel ration strings
rationItemString = "Travelling Ration";
rationLabelString = "Travel Rations:";

-- adding map strings
mapInstructionString =  "Drag your maps from your inventory onto the " ..
                           "quickslot below.  Then press the button " ..
	                        "to set your map home skills.";
mapErrorString = "No maps detected, no updates made.";

-- error messages
resetOrderString = "Your order settings have been reset due to a change in " ..
                  "the number of items saved.  This should only occur once.\n" ..
                  "Sorry for the inconvenience.";
                  
resetEnabledString = "Your selected skills have been reset due to an error " ..
                  "in the number of items previously saved.\n";

-- options window strings
generalTabString = "General";
selectTabString = "Select"; 
sortTabString = "Sort";
hideString = "Hide Travel Window at Startup";
toggleString = "Show Toggle Button";
pulldownTravelString = "Travel when pulldown skill selected";
hideOnCombatString = "Hide window when entering combat";
genericLabelString = "Generic Skills";
racialLabelString = "Racial Skills";
classLabelString = "Class Skills";
repLabelString = "Reputation Skills";
moveUpString = "Move Up";
moveDownString = "Move Down"; 
moveTopString = "Move To Top";
moveBottomString = "Move To Bottom";
toggleSlidersString = "Toggle Button Opacity";
mainSlidersString = "Main Window Opacity";
minString = "Min: ";
maxString = "Max: ";
resetSettingsString = "Reset all to default";

-- menu strings
menuFiltersString = "Filters";
menuGenString = "Show Generic Travels";
menuRaceString = "Show Racial Travels";
menuRepString = "Show Reputation Travels";
menuClassString = "Show Class Travels";
menuModeString = "Mode";
menuTextString = "Text List";
menuIconString = "Icon Grid";
menuCaroString = "Carousel";
menuPullString = "Pulldown List";
menuMapString = "Add Map";   
menuOptionsString = "Options";

--[[ travel skills ]]--
--[[ Add all the travel skills ]]--
TravelDictionaries = class()

function TravelDictionaries:Constructor()

	-- create the indexed tables that store all the data
	genLocations = IndexedDictionary();	
	hunterLocations = IndexedDictionary();
	wardenLocations = IndexedDictionary();
	racialLocations = IndexedDictionary();
	repLocations = IndexedDictionary();
	creepLocations = IndexedDictionary();
	
	-- a table to store the number of each type of travel skill
	travelCount = {0,0,0,0,0,0};
	totalLocations = 0;
	
	self:CreateDictionaries();
   self:GetCounts();	
end

-- function to count the number of skills in total
function TravelDictionaries:GetCounts()
	travelCount[1] = hunterLocations:GetCount();	
	travelCount[2] = wardenLocations:GetCount();
	travelCount[3] = genLocations:GetCount();
	travelCount[4] = repLocations:GetCount();
	travelCount[5] = racialLocations:GetCount();
	travelCount[6] = creepLocations:GetCount();
	
	totalLocations = travelCount[1] + travelCount[2] + travelCount[3] + travelCount[4] + travelCount[5];	
end

-- function to return the count of a specific type of travel
function TravelDictionaries:GetCount(index)
	return travelCount[index];
end

-- function to get the total number of travel skills
function TravelDictionaries:GetTotalCount()
   return totalLocations;
end

-- function to return the 5 tables containing the travel skills
function TravelDictionaries:GetLocations()
	return genLocations, hunterLocations, wardenLocations, racialLocations, repLocations;
end

function TravelDictionaries:CreateDictionaries()
	-- add the data to custom dictionaries to maintain the order

	hunterLocations:AddData("Return to Camp","0x7000A2C1");
	hunterLocations:AddData("Guide to Bree","0x70003F42");
	hunterLocations:AddData("Guide to Thorin's Hall","0x70003F41");
	hunterLocations:AddData("Guide to Michel Delving","0x7000A2C3");
	hunterLocations:AddData("Guide to Esteldín","0x70003F43");
	hunterLocations:AddData("Guide to Evendim","0x7000A2C4");
	hunterLocations:AddData("Guide to Ost Guruth","0x7000A2C2");
	hunterLocations:AddData("Guide to Rivendell","0x70003F44");
	hunterLocations:AddData("Guide to Sûri-Kylä","0x70017C82");
	hunterLocations:AddData("Guide to West Angmar","0x7000A2C5");
	hunterLocations:AddData("Guide to East Angmar","0x7000A2C6");
	hunterLocations:AddData("Guide to Echad Dúnann","0x70017C81");
	hunterLocations:AddData("Guide to Twenty-First Hall","0x70017C7A");
	hunterLocations:AddData("Guide to Mirk-Eaves","0x7001F459");
	hunterLocations:AddData("Guide to Harndirion","0x700235EF");
	hunterLocations:AddData("Guide to Galtrev","0x7002A93F");

	wardenLocations:AddData("Muster in Ost Guruth","0x70014786");
	wardenLocations:AddData("Muster in Esteldín","0x70014798");
	wardenLocations:AddData("Muster in Evendim","0x7001478E");
	wardenLocations:AddData("Muster in Rivendell","0x70014791");	
	wardenLocations:AddData("Muster in Sûri-Kylä","0x700237D4");
	wardenLocations:AddData("Muster in Twenty-First Hall","0x7001819E");
	wardenLocations:AddData("Muster in Mirk-Eaves","0x7001F45C");
	wardenLocations:AddData("Muster in Harndirion","0x700235EB");
   wardenLocations:AddData("Muster in Galtrev","0x7002A90A");					

	racialLocations:AddData("Return to Bree","0x700062F6");
	racialLocations:AddData("Return to Shire","0x700062C8");
	racialLocations:AddData("Return to Thorin's","0x70006346");
	racialLocations:AddData("Return to Rivendell","0x7000631F");	

   -- check if a function that only exists in the newest update exists,
   -- if so, add the Return Home skills
   if(Turbine.UI.Control.IsDisplayed ~= nil) then
	  genLocations:AddData("Return Home 1","0x700256BA");
	  genLocations:AddData("Return Home 2","0x70025792");
	  genLocations:AddData("Return Home 3","0x70025793");
	  genLocations:AddData("Return Home 4","0x70025794");
	  genLocations:AddData("Return Home 5","0x70025795");
	  genLocations:AddData("Return Home 6","0x70025796");
	end
	genLocations:AddData("Personal House","0x7000D046");
	genLocations:AddData("Kinship House","0x7000D047");

	repLocations:AddData("Return to Thorin's (Rep)","0x7001BF91");
	repLocations:AddData("Return to Bree (Rep)","0x7001BF90");
	repLocations:AddData("Return to Shire (Rep)","0x70023262");
	repLocations:AddData("Return to Rivendell (Rep)","0x70023263");
	repLocations:AddData("Return to Ost Guruth (Rep)","0x70020441");
	repLocations:AddData("Return to Mirkwood (Rep)","0x7001F374");
	repLocations:AddData("Return to Enedwaith (Rep)","0x70021FA2");
	                           
	creepLocations:AddData("Crude Map to Isendeep","0x70028BB3");
	creepLocations:AddData("Crude Map to Tol Ascarnen","0x70028BB6");
	creepLocations:AddData("Crude Map to Lugazag","0x70028BB7");
	creepLocations:AddData("Crude Map to Gramsfoot","0x70028BBC");
	creepLocations:AddData("Crude Map to Tirith Rhaw","0x70028BBE");
	creepLocations:AddData("Crude Map to Grimwood","0x70028BBF");
	creepLocations:AddData("Poor Map to Isendeep","0x70028BAF");
	creepLocations:AddData("Poor Map to Tol Ascarnen","0x70028BB1");
	creepLocations:AddData("Poor Map to Lugazag","0x70028BB2");
	creepLocations:AddData("Poor Map to Tirith Rhaw","0x70028BB4");
	creepLocations:AddData("Poor Map to Grimwood","0x70028BB9");
	creepLocations:AddData("Good Map to Isendeep","0x70028BC0");
	creepLocations:AddData("Good Map to Tol Ascarnen","0x70028BC2");
	creepLocations:AddData("Good Map to Lugazag","0x70028BB5");                                     
	creepLocations:AddData("Map to Gramsfoot","0x70028BC1");
	creepLocations:AddData("Good Map to Tirith Rhaw","0x70028BB0");
	creepLocations:AddData("Good Map to Grimwood","0x70028BBD");

end


--[[ use a modified table that can maintain a key value for each entry ]]--
IndexedDictionary = class() 

function IndexedDictionary:Constructor()

   -- set default tables
	self.listOfKeys = { };
	self.dataPairs = { };
	self.numberOfPairs = 0;
end

-- function to get the total number of entries in the dictionary
function IndexedDictionary:GetCount()
	return self.numberOfPairs;
end

-- function to add a key/data pair to the end of the dictionary
function IndexedDictionary:AddData(key, data)
   -- increase the counter
	self.numberOfPairs = self.numberOfPairs + 1;

   -- insert the key into the list of keys
	table.insert(self.listOfKeys,key);
	-- add the key/data into the table
	self.dataPairs[key] = data;

end

-- function to add a key/data pair at a specific index
function IndexedDictionary:InsertData(index, key, data)
   -- increase the counter
	self.numberOfPairs = self.numberOfPairs + 1;

   -- add the key to the list of keys at the specified index
	table.insert(self.listOfKeys, index, key);
	-- add the key/data into the table
	self.dataPairs[key] = data;
end

-- function to remove a key/data pair at an index
function IndexedDictionary:RemoveAtIndex(index)
	
	self.location = 0;
	self.counter = 0;

   -- loop through the main table	
	for k,v in pairs(self.dataPairs) do
		self.counter = self.counter + 1;
		
		-- if the keys match, set the location to remove from the table
		if(k == self.listOfKeys[index]) then
			self.location = self.counter;
		end
	end

   -- remove the item and the key from the tables
	table.remove(self.dataPairs,self.location);
	table.remove(self.listOfKeys,index);

   -- decrease the counters
	self.numberOfPairs = self.numberOfPairs - 1;
end

-- function to get the key at a specified index
function IndexedDictionary:KeyAtIndex(index)
	local string = self.listOfKeys[index];
	return string;
end

-- function to return the data at a specified index
function IndexedDictionary:DataAtIndex(index)
	local string = self.listOfKeys[index];
	local data = self.dataPairs[string];

	return data;
end

-- function to return the data pair at an index
function IndexedDictionary:PairAtIndex(index)
	local string = self.listOfKeys[index];
	local data = self.dataPairs[string];

	return string,data;
end

-- function to get the data by a key value
function IndexedDictionary:DataByKey(key)
	return self.dataPairs[key];
end

-- function to find the index value based on a key
function IndexedDictionary:IndexByKey(key)

   -- return if the list of keys is empty
	if(self.listOfKeys == { }) then
		return -1;
	end

   -- loop through the list of keys and search for the key value
	for i = 1,table.getn(self.listOfKeys),1 do
	   -- return the index value if a match is made
		if(self.listOfKeys[i] == key) then
			return i;
		end
	end
	
	-- return -1 since we did not find a key
	return -1;
end


